﻿// Copyright (c) 2014, Leandro Toss Hoffmann (INPE)> <leandro.hoffmann@inpe.br >
// Permission to use, copy, modify, and distribute this model for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
// This model is provided "as is" and the author disclaims all warranties with regard to it. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using EA;

namespace SMP2CodeGen
{
    class LibGenClass
    {
        
        string path = "";
        string sbPath = "";
        EA.Repository Repository;

        public void GenerateLib(EA.Repository a_Repository, string Location)
        {
            Repository = a_Repository;

            if (Repository.GetTreeSelectedItemType() != EA.ObjectType.otPackage)
            {
                MessageBox.Show("Please, select a Package from the Tree View");
                return;
            }

            // Gets the path to write files
            CodeGenDialog dialog = new CodeGenDialog();
            DialogResult result = dialog.ShowDialog();
            if (result == DialogResult.OK)
                path = dialog.GetPath();
            else
                return;

            // Generate several files
            if (Repository.GetTreeSelectedItemType() == EA.ObjectType.otPackage)
            {
                sbPath = Microsoft.VisualBasic.Interaction.InputBox("Inform the Simubox root path", "Include Path",Repository.GetTreeSelectedPackage().CodePath);
                sbPath = sbPath.Replace("\\", "/");
                GenerateProjectFile(Repository.GetTreeSelectedPackage());
                GenerateGlobalFile(Repository.GetTreeSelectedPackage());
                GenerateDefinitionFile(Repository.GetTreeSelectedPackage());
                GenerateImplementationFile(Repository.GetTreeSelectedPackage());
                GenerateSMPPackage(Repository.GetTreeSelectedPackage());
            }
        }
       
        private void GenerateProjectFile(Package package)
        {
            System.IO.StreamWriter file = new System.IO.StreamWriter(@path + "\\" + package.Name + ".pro", false, Encoding.GetEncoding(1252));

            file.WriteLine("QT\t-= gui");
            file.WriteLine("");
            file.WriteLine("TARGET = " + package.Name);
            file.WriteLine("TEMPLATE = lib");
            file.WriteLine("");
            file.WriteLine("DEFINES += " + package.Name.ToUpper() + "_LIBRARY");
            file.WriteLine("");
            file.WriteLine("INCLUDEPATH += " + sbPath + "/incl");
            file.WriteLine("");
            file.WriteLine("HEADERS += " + package.Name + "_global.h \\");
            foreach (Element element in package.Elements)
            {
                if ((element.Type == "Class" && element.Stereotype == "SMP2Model")
                    || element.Type == "Interface")
                    file.WriteLine("\t" + element.Name + ".h\\");
            }
            file.WriteLine("\t" + package.Name + "Lib.h");
            file.WriteLine("SOURCES += \\");
            foreach (Element element in package.Elements)
            {
                if (element.Type == "Class" && element.Stereotype == "SMP2Model")
                    file.WriteLine("\t" + element.Name + ".cpp\\");
            }
            file.WriteLine("\t" + package.Name + "Lib.cpp");
            file.WriteLine("");
            file.WriteLine("LIBS += -L" + sbPath + "/build/simuboxLib/debug -lsimuboxLib");
            file.WriteLine("LIBS += -L" + sbPath + "/build/SmpLib/debug -lSmpLib");
            file.WriteLine("");
            file.Close();
        }
        private void GenerateGlobalFile(Package package)
        {
            System.IO.StreamWriter file = new System.IO.StreamWriter(@path + "\\" + package.Name + "_global.h", false, Encoding.GetEncoding(1252));
            file.WriteLine("#ifndef " + package.Name.ToUpper() + "_GLOBAL_H");
            file.WriteLine("#define " + package.Name.ToUpper() + "_GLOBAL_H");
            file.WriteLine("");
            file.WriteLine("#include <QtCore/qglobal.h>");
            file.WriteLine("");
            file.WriteLine("#if defined(" + package.Name.ToUpper() + "_LIBRARY)");
            file.WriteLine("#  define " + package.Name.ToUpper() + "SHARED_EXPORT Q_DECL_EXPORT");
            file.WriteLine("#else");
            file.WriteLine("#  define " + package.Name.ToUpper() + "SHARED_EXPORT Q_DECL_IMPORT");
            file.WriteLine("#endif");
            file.WriteLine("");
            file.WriteLine("#endif // " + package.Name.ToUpper() + "_GLOBAL_H");
            file.WriteLine("");
            file.Close();
        }
        private void GenerateDefinitionFile(Package package)
        {
            System.IO.StreamWriter file = new System.IO.StreamWriter(@path + "\\" + package.Name + "Lib.h", false, Encoding.GetEncoding(1252));
            file.WriteLine("#ifndef " + package.Name.ToUpper() + "LIB_H");
            file.WriteLine("#define " + package.Name.ToUpper() + "LIB_H");
            file.WriteLine("");
            file.WriteLine("#include \"" + package.Name + "_global.h\"");
            file.WriteLine("#include \"Smp/ISimulator.h\"");
            file.WriteLine("#include \"Smp/IPublication.h\"");
            file.WriteLine("");
            file.WriteLine("extern \"C\"");
            file.WriteLine("{");
            file.WriteLine("\t/**");
            file.WriteLine("\t * Initialise function for dynamic package.");
            file.WriteLine("\t * @param simulator simulation environment reference");
            file.WriteLine("\t * @param typeRegistry type register reference");
            file.WriteLine("\t */");
            file.WriteLine("\t " + package.Name.ToUpper() + "SHARED_EXPORT bool Initialise(Smp::ISimulator* simulator,");
            file.WriteLine("\t\t\tSmp::Publication::ITypeRegistry* typeRegistry);");
            file.WriteLine("");
            file.WriteLine("\t/**");
            file.WriteLine("\t * Finalise function for dynamic package.");
            file.WriteLine("\t */");
            file.WriteLine("\t" + package.Name.ToUpper() + "SHARED_EXPORT bool Finalise();");
            file.WriteLine("}");
            file.WriteLine("");
            file.WriteLine("#endif // " + package.Name.ToUpper() + "LIB_H");
            file.WriteLine("");
            file.Close();
        }
        private void GenerateImplementationFile(Package package)
        {
            System.IO.StreamWriter file = new System.IO.StreamWriter(@path + "\\" + package.Name + "Lib.cpp", false, Encoding.GetEncoding(1252));

            file.WriteLine("#include \"" + package.Name + "Lib.h\"");
            file.WriteLine("");
            foreach(Element element in package.Elements)
            {
                if((element.Type == "Class" && element.Stereotype == "SMP2Model")
                    || element.Type == "Interface")
                    file.WriteLine("#include \"" + element.Name + ".h\"");
            }
            file.WriteLine("#include \"Smp/Management/IManagedSimulator.h\"");
            file.WriteLine("#include \"Sb/Factory.h\"");
            file.WriteLine("");
            file.WriteLine("#include <vector>");
            file.WriteLine("");

            file.WriteLine("/**");
            file.WriteLine(" * vector to store the factory objects until the end of DLL use.");
            file.WriteLine(" */");
            file.WriteLine("std::vector<Smp::IFactory*> m_factories;");
            file.WriteLine("");

            file.WriteLine("/**");
            file.WriteLine(" * Initialise function for dynamic package.");
            file.WriteLine(" * @param simulator simulation environment reference");
            file.WriteLine(" * @param typeRegistry type register reference");
            file.WriteLine(" */");
            file.WriteLine("bool Initialise(Smp::ISimulator* simulator,");
            file.WriteLine("\t\tSmp::Publication::ITypeRegistry* typeRegistry)");
            file.WriteLine("{");
            file.WriteLine("\tSmp::Management::IManagedSimulator *sim = dynamic_cast<Smp::Management::IManagedSimulator*>(simulator);");
            file.WriteLine("\tif(!sim)");
            file.WriteLine("\t\treturn false;");
            file.WriteLine("");
            foreach(Element element in package.Elements)
            {
                if(element.Type == "Class" && element.Stereotype == "SMP2Model")
                {
                    file.WriteLine("\t// " + package.Name + "::" + element.Name + " = " + element.ElementGUID);
                    file.WriteLine("\tSB::Factory< " + package.Name + "::" + element.Name + " >* factory" + element.Name);
                    file.WriteLine("\t\t= new SB::Factory< " + package.Name + "::" + element.Name + " >(\"" + element.Name + "\", \"Factory for " + element.Name + "\", " + package.Name + "::Uuid_" + element.Name + ", " + package.Name + "::Uuid_" + element.Name + ");");
                    file.WriteLine("\tm_factories.push_back(factory" + element.Name  +");");
                    file.WriteLine("\tsim->RegisterFactory(factory" + element.Name + ");");
                    file.WriteLine("");
                }
            }
            
            file.WriteLine("\treturn true;");
            file.WriteLine("}");
            file.WriteLine("");

            file.WriteLine("/**");
            file.WriteLine(" * Finalise function for dynamic package.");
            file.WriteLine(" */");
            file.WriteLine("bool Finalise()");
            file.WriteLine("{");
            file.WriteLine("\tif(!m_factories.size())");
            file.WriteLine("\t\treturn false;");
            file.WriteLine("");
            file.WriteLine("\tfor(unsigned int i=0; i < m_factories.size(); i++)");
            file.WriteLine("\t\tdelete(m_factories[i]);");
            file.WriteLine("");
            file.WriteLine("\tm_factories.clear();");
            file.WriteLine("");
            file.WriteLine("\treturn true;");
            file.WriteLine("}");
            file.WriteLine("");

            file.Close();
        }
        private void GenerateSMPPackage(Package package)
        {
            System.IO.StreamWriter file = new System.IO.StreamWriter(@path + "\\" + package.Name + ".pck", false, Encoding.GetEncoding(1252));
            string datetime = DateTime.Now.ToString("yyyy-MM-ddTHH:mm:ss+HH:mm");

            file.WriteLine("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
            file.WriteLine("<Package:Package Id=\"" + package.Name + "Pck\" Name=\"" + package.Name + "Pck\" Date=\"" + datetime + "\" Creator=\"Simubox's SMP2 Package Generator\" Version=\"1.0\"");
            file.WriteLine("  xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"");
            file.WriteLine("  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ");
            file.WriteLine("  xmlns:xlink=\"http://www.w3.org/1999/xlink\" ");
            file.WriteLine("  xmlns:Types=\"http://www.esa.int/2008/02/Core/Types\" ");
            file.WriteLine("  xmlns:Elements=\"http://www.esa.int/2008/02/Core/Elements\" ");
            file.WriteLine("  xmlns:Catalogue=\"http://www.esa.int/2008/02/Smdl/Catalogue\"");
            file.WriteLine("  xmlns:Package=\"http://www.esa.int/2008/02/Smdl/Package\" >");
            foreach (Element element in package.Elements)
            {
                if (element.Type == "Class" && element.Stereotype == "SMP2Model")
                {
                    file.WriteLine("  <Implementation Uuid=\"" + element.ElementGUID.TrimStart('{').TrimEnd('}') + "\">");
                    file.WriteLine("    <Type xlink:title=\"" + element.Name + "\" xlink:href=\"./" + package.Name + ".cat#" + package.Name + "." + element.Name + "\"/>");
                    file.WriteLine("  </Implementation>");
                }
            }
            file.WriteLine("</Package:Package>");
            
            file.Close();
        }
    }
}
