﻿///////////////////////////////////////////////////////////////////////////////
//  INSTITUTO NACIONAL DE PESQUISAS ESPACIAIS
//  Simubox - SMP2 Model Development Kit
///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2014, Leandro Toss Hoffmann (INPE)> <leandro.hoffmann@inpe.br >
// Permission to use, copy, modify, and distribute this model for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
// This model is provided "as is" and the author disclaims all warranties with regard to it. 
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using EA;

namespace SMP2CodeGen
{
    public class SMP2CodeGenClass
    {
	// define menu constants
        const string menuHeader = "-&Simubox";
        const string menuCodeGen = "&Generate C++ implmentation code...";
        const string menuCatalogueGen = "&Generate SMP2 catalogue...";
        const string menuAssemblyGen = "&Generate SMP2 assembly...";
        const string menuScheduleGen = "&Generate SMP2 schedule...";
        const string menuLibGen = "&Generate component library...";
 

        ///
        /// Called Before EA starts to check Add-In Exists
        /// Nothing is done here.
        /// This operation needs to exists for the addin to work
        ///
        /// <param name="Repository" />the EA repository
        /// a string
        public String EA_Connect(EA.Repository Repository)
        {
            //No special processing required.
            // return "MDG" if this is a special MDG Addin
            return "";
        }
 
        ///
        /// Called when user Clicks Add-Ins Menu item from within EA.
        /// Populates the Menu with our desired selections.
        /// Location can be "TreeView" "MainMenu" or "Diagram".
        ///
        /// <param name="Repository" />the repository
        /// <param name="Location" />the location of the menu
        /// <param name="MenuName" />the name of the menu
        ///
        public object EA_GetMenuItems(EA.Repository Repository, string Location, string MenuName)
        {
 
                switch (MenuName)
                {
                    // defines the top level menu option
                    case "":
                        return menuHeader;
                    // defines the submenu options
                    case menuHeader:
                        string[] subMenus = { menuCodeGen, menuCatalogueGen, menuAssemblyGen, menuScheduleGen, menuLibGen};
                        return subMenus;
                }
 
            return "";
        }
 
        ///
        /// returns true if a project is currently opened
        ///
        /// <param name="Repository" />the repository
        /// true if a project is opened in EA
        bool IsProjectOpen(EA.Repository Repository)
        {
            try
            {
                EA.Collection c = Repository.Models;
                return true;
            }
            catch
            {
                return false;
            }
        }
 
        ///
        /// Called once Menu has been opened to see what menu items should active.
        ///
        /// <param name="Repository" />the repository
        /// <param name="Location" />the location of the menu
        /// <param name="MenuName" />the name of the menu
        /// <param name="ItemName" />the name of the menu item
        /// <param name="IsEnabled" />boolean indicating whethe the menu item is enabled
        /// <param name="IsChecked" />boolean indicating whether the menu is checked
        public void EA_GetMenuState(EA.Repository Repository, string Location, string MenuName, string ItemName, ref bool IsEnabled, ref bool IsChecked)
        {
            IsEnabled = false;

            if (IsProjectOpen(Repository))
            {
                switch (ItemName)
                {
                    // define the state of the CodeGen menu option
                    case menuCodeGen:
                        // Checks if the elements is a class for code generation
                        if (Repository.GetTreeSelectedItemType() == EA.ObjectType.otElement
                            || Repository.GetTreeSelectedItemType() == EA.ObjectType.otPackage)
                            IsEnabled = true;
                        break;
                    // define the state of the CatalogueGen menu option
                    case menuCatalogueGen:
                        // Checks if the elements is a package for code generation
                        if (Repository.GetTreeSelectedItemType() == EA.ObjectType.otPackage && Location == "TreeView")
                            IsEnabled = true;
                        break;
                    // define the state of the AssemblyGen menu option
                    case menuAssemblyGen:
                        // Checks if the elements is a package for code generation
                        if (Location == "Diagram")
                            IsEnabled = true;
                        break;
                    // define the state of the Schedule Gen menu option
                    case menuScheduleGen:
                        // Checks if the elements is a package for code generation
                        if (Location == "Diagram")
                            IsEnabled = true;
                        break;
                    // define the state of the LibGen menu option
                    case menuLibGen:
                        // Checks if the elements is a package for code generation
                        if (Repository.GetTreeSelectedItemType() == EA.ObjectType.otPackage && Location == "TreeView")
                            IsEnabled = true;
                        break;
                    default:
                        IsEnabled = false;
                        break;
                }
            }
        }
 
        ///
        /// Called when user makes a selection in the menu.
        /// This is your main exit point to the rest of your Add-in
        ///
        /// <param name="Repository" />the repository
        /// <param name="Location" />the location of the menu
        /// <param name="MenuName" />the name of the menu
        /// <param name="ItemName" />the name of the selected menu item
        public void EA_MenuClick(EA.Repository Repository, string Location, string MenuName, string ItemName)
        {
            switch (ItemName)
            {
                // user has clicked the menuCodeGen menu option
                case menuCodeGen:
                    CppGenClass CodeGen = new CppGenClass();
                    CodeGen.GenerateCode(Repository, Location);
                    break;
                // user has clicked the menuAssemblyGen menu option
                case menuAssemblyGen:
                    AssemblyGenClass asbGen = new AssemblyGenClass();
                    asbGen.GenerateAssembly(Repository, Location);
                    break;
                // user has clicked the menuScheduleGen menu option
                case menuScheduleGen:
                    ScheduleGenClass schGen = new ScheduleGenClass();
                    schGen.GenerateSchedule(Repository, Location);
                    break;
                case menuCatalogueGen:
                    MessageBox.Show("This function is not implemented yet!");    
                    break;
                // user has clicked the menuLibGen menu option
                case menuLibGen:
                    LibGenClass LibGen = new LibGenClass();
                    LibGen.GenerateLib(Repository, Location);
                    break;
            }
        }
 
        ///
        /// EA calls this operation when it exists. Can be used to do some cleanup work.
        ///
        public void EA_Disconnect()
        {
            GC.Collect();
            GC.WaitForPendingFinalizers();
        }


        ///
        /// Operation to extract input and output Port data from SMP2 Models.
        /// This operation is to be called from SMP2 Code Generatio Template, using the Macro EXEC_ADD_IN("SMP2CodeGen","GetPortDeclaration", classGUID, portType)
        /// The portType can be 'input' or 'output'
        public object GetPortDeclaration(EA.Repository Repository, object argList)
        {
            // Search for class using its GUID
            
            Element e = UtilClass.GetClassByGuid(Repository, argList);
            if (e == null)
                return "";

            // Get port type (input or output)
            string portType = "";
            try
            {
                string[] stringList = argList as string[];
                portType = stringList[1];
                if (portType == "" || portType == null)
                    throw new System.Exception();
            } catch
            {
                MessageBox.Show("Error when invoking Addin service: classGUID argument not present as string. Expected 'EXEC_ADD_IN(\"SMP2CodeGen\",\"GetPortDeclaration\", classGUID, portType'");
                return "";
            }

            // Generated code
            string code = "";

            // Search for embbeded elements of type Port
            foreach (Element port in e.Elements)
            {
                // Only considerer input/output fields those ports without  exposed interfaces
                if (port.Type == "Port" && port.EmbeddedElements.Count == 0)
                {
                    string fieldType = "";
                    string fieldName = "";
                    string upperBound = "";
                    foreach (TaggedValue tag in port.TaggedValues)
                    {
                        if (tag.Name == "type")
                            fieldType = tag.Value;
                        else if (tag.Name == portType && tag.Value == "True")
                            fieldName = port.Name;
                        //else if (tag.Name == "upperBound")
                            //upperBound = tag.Value;
                    }
                    if(fieldType != "" && fieldName != "")
                    {
                        code += "\n\t/// " + port.Notes;

                        if (port.Multiplicity != "")
                        {
                            string[] mult = port.Multiplicity.Replace("..", "#").Split('#');
                            if (mult.Count() > 1)
                                upperBound = mult[1].TrimEnd(']');
                        }
                        if(upperBound == "")
                            code += "\n\tpublic: " + fieldType + " " + fieldName + ";";
                        else
                            code += "\n\tpublic: " + fieldType + " " + fieldName + "[" + upperBound + "];";
                    }
                                        
                }
                
            }
            return code;
        }

        ///
        /// Operation to extract input and output Port data from SMP2 Models and identify Required interfaces, transforming then in includes
        /// This operation is to be called from SMP2 Code Generatio Template, using the Macro EXEC_ADD_IN("SMP2CodeGen","GetReferenceImports", classGUID)
        public object GetReferenceImports(EA.Repository Repository, object argList)
        {
            // Search for class using its GUID
            Element e = UtilClass.GetClassByGuid(Repository, argList);
            if (e == null)
                return "";

            // Generated code
            string code = "";

            // Search for embbeded elements of type Port
            foreach (Element port in e.Elements)
            {
                // Only considerer ports with Required SMP2interfaces
                if (port.Type == "Port" && port.EmbeddedElements.Count > 0 && port.Stereotype == "SMP2interface")
                {
                    foreach (Element expInterface in port.EmbeddedElements)
                    {
                        if (expInterface.Type == "RequiredInterface")
                            code += "\n#include \"" + expInterface.Name + ".h\"";
                    }
                }
            }
            if(code != "")
                code = "#include \"Sb/Management/ManagedReference.h\"" + code;

            return code;
        }
        ///
        /// Operation to extract container names defined in composition links in order to write include headers.
        /// This operation is to be called from SMP2 Code Generatio Template, using the Macro EXEC_ADD_IN("SMP2CodeGen","GetContainerImports", classGUID)
        public object GetContainerImports(EA.Repository Repository, object argList)
        {
            // Search for class using its GUID
            Element e = UtilClass.GetClassByGuid(Repository, argList);
            if (e == null)
                return "";

            // Generated code
            string code = "";
            
            // Search for composite connector
            foreach (Connector connector in e.Connectors)
            {
                if (connector.Type == "Aggregation" && connector.Stereotype == "SMP2container")
                {
                    Element client = Repository.GetElementByID(connector.ClientID);
                    if ((client.Type != "Class" && client.Type != "Interface") || client.Name == e.Name)
                        continue;
                    code += "\n#include \"" + client.Name + ".h\"";
                }
            }
            if (code != "")
                code = "#include \"Sb/Management/ManagedContainer.h\"" + code;
            return code;
        }
        ///
        /// Operation to extract the implemented interfaces from a SMP2 Model.
        /// They are expected to be defined as exposed interfaces of Model's embbeded ports
        /// This operation is to be called from SMP2 Code Generatio Template, using the Macro EXEC_ADD_IN("SMP2CodeGen","GetImplInterfaces", classGUID, placeHolder)
        /// placeHolder must contain a substring "<interface>" which will be replaced by the actual name of the interface.
        public object GetImplInterfaces(EA.Repository Repository, object argList)
        {
            // Search for class using its GUID
            Element e = UtilClass.GetClassByGuid(Repository, argList);
            if (e == null)
                return "";
            string placeHolder = "";
            try
            {
                string[] args = argList as string[];
                if (args.Count() >= 2)
                    placeHolder = args[1];
            }
            catch { MessageBox.Show("Argument list '<interface>' not found in 'GetImplInterfaces'."); return ""; }

            placeHolder = placeHolder.Replace("\\n", "\n").Replace("\\\"", "\"");
            // Generated code
            string code = "";

            // Search for embbeded elements of type Port
            foreach (Element port in e.Elements)
            {
                // Only considerer ports with Required SMP2interfaces
                if (port.Type == "Port" && port.EmbeddedElements.Count > 0 && port.Stereotype == "SMP2interface")
                {
                    foreach (Element expInterface in port.EmbeddedElements)
                        if (expInterface.Type == "ProvidedInterface")
                            code += placeHolder.Replace("<interface>", expInterface.Name);
                }
            }
            return code;
        }
        ///
        /// Operation to extract input and output Port data from SMP2 Models and identify Required interfaces, transforming then in References.
        /// This operation is to be called from SMP2 Code Generatio Template, using the Macro EXEC_ADD_IN("SMP2CodeGen","GetReferenceDeclaration", classGUID)
        public object GetReferenceDeclaration(EA.Repository Repository, object argList)
        {
            // Search for class using its GUID
            Element e = UtilClass.GetClassByGuid(Repository, argList);
            if (e == null)
                return "";

            // Generated code
            string code = "";

            // Search for embbeded elements of type Port
            foreach (Element port in e.Elements)
            {
                // Only considerer ports with Required SMP2interfaces
                if (port.Type == "Port" && port.EmbeddedElements.Count > 0 && port.Stereotype == "SMP2interface")
                {
                    foreach (Element expInterface in port.EmbeddedElements)
                    {
                        if (expInterface.Type == "RequiredInterface")
                        {
                            code += "\n\t/// " + expInterface.Notes;
                            code += "\n\tprotected: SB::Management::ManagedReference< " + expInterface.Name + " >* _" + port.Name + ";";
                            code += "\n\tprotected: Smp::Management::IManagedReference* " + port.Name + ";";
                        }
                    }
                }

            }
            return code;
        }
        ///
        /// Operation to extract the implemented interfaces from a SMP2 Model and their respective virtual operations
        /// They are expected to be defined as exposed interfaces of Model's embbeded ports
        /// This operation is to be called from SMP2 Code Generatio Template, using the Macro EXEC_ADD_IN("SMP2CodeGen","GetInterfaceDefinitions", classGUID)
        public object GetInterfaceDefinitions(EA.Repository Repository, object argList)
        {
            // Search for class using its GUID
            Element e = UtilClass.GetClassByGuid(Repository, argList);
            if (e == null)
                return "";

            string code = "";

            // Search for embbeded elements of type Port
            foreach (Element port in e.Elements)
            {
                // Only considerer ports with Required SMP2interfaces
                if (port.Type != "Port" || port.EmbeddedElements.Count <= 0 || port.Stereotype != "SMP2interface")
                    continue;
                foreach (Element expInterface in port.EmbeddedElements)
                {
                    if (expInterface.Type != "ProvidedInterface")
                        continue;
                    // Search for provided interface in the current package
                    Package currPpackage = Repository.GetPackageByID(e.PackageID);
                    foreach(Element model in currPpackage.Elements)
                    {
                        if (model.Name != expInterface.Name)
                            continue;
                        foreach (Method op in model.Methods)
                        {
                            if (!op.IsPure)
                                continue;
                            
                            code += "\n\t/**";
                            code += "\n\t  * Implementation of base method " + expInterface.Name + "::" + op.Name;
                            code += "\n\t  */";
                            code += "\n\tvirtual " + op.ReturnType + " " + op.Name + "(";
                            foreach (Parameter param in op.Parameters)
                                code += (param.IsConst ? "const " : "") + param.Type + " " + param.Name + ",";
                            code = code.TrimEnd(',');
                            code += ")" + (op.IsConst ? " const" : "") + ";";
                                            
                        }
                    }

                }
                            
            }
            return code;
        }
    } // end class
} // end namespace
