﻿// Copyright (c) 2014, Leandro Toss Hoffmann (INPE)> <leandro.hoffmann@inpe.br >
// Permission to use, copy, modify, and distribute this model for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
// This model is provided "as is" and the author disclaims all warranties with regard to it. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using EA;

namespace SMP2CodeGen
{
    class ScheduleGenClass
    {
        string path = "";
        EA.Repository Repository;
        Diagram diagram;
        System.IO.StreamWriter file;

        public void GenerateSchedule(EA.Repository a_Repository, string Location)
        {
            Repository = a_Repository;
            diagram = Repository.GetCurrentDiagram();

            if (diagram == null)
            {
                MessageBox.Show("Please, select a Diagram from the Tree View");
                return;
            }

            // Gets the path to write files
            CodeGenDialog dialog = new CodeGenDialog();
            DialogResult result = dialog.ShowDialog();
            if (result == DialogResult.OK)
                path = dialog.GetPath();
            else
                return;

            file = new System.IO.StreamWriter(@path + "\\" + diagram.Name.Trim() + ".sch", false);

            String idents = "";

            // Assembly Header
            string datetime = DateTime.Now.ToString("yyyy-MM-ddTHH:mm:ss+HH:mm");
            file.WriteLine("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
            file.WriteLine("<Schedule:Schedule Name=\"" + diagram.Name.Trim() + "_Sch\" Date=\"" + datetime + "\"  Name=\"" + diagram.Name.Trim() + "_Sch\"");
            file.WriteLine("Creator=\"Simubox's SMP2 Assembly Generator\" Version=\"1.0\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"");
            file.WriteLine("xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
            file.WriteLine("xmlns:xlink=\"http://www.w3.org/1999/xlink\"");
            file.WriteLine("xmlns:Types=\"http://www.esa.int/2008/02/Core/Types\"");
            file.WriteLine("xmlns:Elements=\"http://www.esa.int/2008/02/Core/Elements\"");
            file.WriteLine("xmlns:Schedule=\"http://www.esa.int/2008/02/Smdl/Schedule\">");
            idents += ("\t");
            file.WriteLine("");
            file.WriteLine(idents + "<!-- EDIT HERE -->");
            file.WriteLine(idents + "<EpochTime>2002-10-10T12:00:00-05:00</EpochTime>");
            file.WriteLine(idents + "<Task Id=\"UpdateTask\" Name=\"UpdateTask\">");
            idents += ("\t");
            file.WriteLine(idents + "<Description>Update Task</Description>");
            
            foreach (EA.DiagramObject obj in diagram.DiagramObjects)
            {
                Element element = Repository.GetElementByID(obj.ElementID);

                // Checks if the model is eligible to be a root instance
                // Is a object instance, has a name and is not contained by other objet
                if (element.Type != "Object" || element.Name == "")
                    continue;

                // Class element of instantiated object
                Element elementClass = Repository.GetElementByID(element.ClassifierID);
                ProcessModelInstance(element, elementClass, idents); 
            }
            idents = idents.Remove(0, 1);
            file.WriteLine(idents + "</Task>");
            file.WriteLine(idents + "<Event xsi:type=\"Schedule:SimulationEvent\" Id=\"CyclicalEvent1\" SimulationTime=\"PT0S\" Name=\"CyclicalEvent1\" CycleTime=\"PT1.0S\" Repeat=\"-1\" >");
            idents += ("\t");
            file.WriteLine(idents + "<Task xlink:title=\"UpdateTask\" xlink:href=\"#UpdateTask\"/>");
            idents = idents.Remove(0, 1);
            file.WriteLine(idents + "</Event>");
            idents = idents.Remove(0, 1);
            file.WriteLine(idents + "</Schedule:Schedule>");

            file.Close();
        }
        private void ProcessModelInstance(Element element, Element elementClass, String idents)
        {

            file.WriteLine("");
            file.WriteLine(idents + "<!-- Entry point and Field links transfer for [" + element.Name + "] -->");
            
            ProcessFieldLinks(element, elementClass, idents);

            file.WriteLine(idents + "<Activity xsi:type=\"Schedule:Trigger\" Id=\"" + element.Name + "Refresh\" Name=\"" + element.Name + "Refresh\">");
            idents += ("\t");
            file.WriteLine(idents + "<Provider xlink:title=\"" + element.Name + "\" xlink:href=\"./somePath.asb#" + elementClass.Name +"\"/>");
            file.WriteLine(idents + "<EntryPoint xlink:title=\"Update\" xlink:href=\"./somePath.cat#" + elementClass.Name + ".Update\"/>");
            idents = idents.Remove(0, 1);
            file.WriteLine(idents + "</Activity>");
        }
        
        private void ProcessFieldLinks(Element element, Element elementClass, String idents)
        {
            foreach (Element port in element.EmbeddedElements)
            {
                // Only considerer ports without exposed interfaces
                if (port.Type != "Port" || port.EmbeddedElements.Count > 0 || port.Stereotype == "SMP2interface")
                    continue;

                //int fieldId = 0;
                foreach (Connector conn in port.Connectors)
                {
                    if (conn.Stereotype != "SMP2fieldLink")
                        continue;
                    // Gets the reference to the source's port
                    Element outputField = Repository.GetElementByID(conn.ClientID); // EA names the source as client, but in this case, the client is the output field
                    if (outputField == null || outputField.ParentID == 0 || outputField.Type != "Port" || outputField.Name == port.Name)
                        continue;

                    // And finally gets the reference to the port's object (Object)
                    Element source = Repository.GetElementByID(outputField.ParentID);
                    if (source == null || source.Type != "Object")
                        continue;

                    string instanceName = element.Name;
                    string inputFieldName = port.Name;
                    file.WriteLine(idents + "<Activity xsi:type=\"Schedule:Transfer\" Id=\"Transfer." + instanceName + "_" + inputFieldName + "\" Name=\"Transfer_" + instanceName + "_" + inputFieldName + "\">");
                    idents += ("\t");
                    file.WriteLine(idents + "<FieldLink xlink:title=\"" + instanceName + "_" + inputFieldName + "\" xlink:href=\"./somePath.asb#" + instanceName + "_" + inputFieldName + "\" />");
                    idents = idents.Remove(0, 1);
                    file.WriteLine(idents + "</Activity>");
                }
            }
        }
      

    } // end class
} //  end namespace
