﻿///////////////////////////////////////////////////////////////////////////////
//  INSTITUTO NACIONAL DE PESQUISAS ESPACIAIS
//  Simubox - SMP2 Model Development Kit
///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2014, Leandro Toss Hoffmann (INPE)> <leandro.hoffmann@inpe.br >
// Permission to use, copy, modify, and distribute this model for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
// This model is provided "as is" and the author disclaims all warranties with regard to it. 
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using EA;

namespace SMP2CodeGen
{
    public class UtilClass
    {
        ///
        /// Operation to extract input and output Port data from SMP2 Models.
        /// This operation is to be called from SMP2 Code Generatio Template, using the Macro EXEC_ADD_IN("SMP2CodeGen","GetReferenceDeclaration", classGUID)
        public static Element GetClassByGuid(EA.Repository Repository, object argList)
        {
            // Search for class using its GUID
            string[] stringList = argList as string[];
            if (stringList == null)
            {
                MessageBox.Show("Error when invoking Addin service: classGUID argument not present as string[] type. Expected 'EXEC_ADD_IN(\"SMP2CodeGen\",\"Func\", classGUID'");
                return null;
            }
            string classGUID = "";
            try
            {
                classGUID = stringList[0];
                if (classGUID == "" || classGUID == null)
                    throw new System.Exception();
            }
            catch
            {
                MessageBox.Show("Error when invoking Addin service: classGUID argument not present as string. Expected 'EXEC_ADD_IN(\"SMP2CodeGen\",\"Func\", classGUID'");
                return null;
            }

            Element e = Repository.GetElementByGuid(classGUID);
            if (e.Type != "Class")
                return null;

            return e;
        }

        public static string GetPublicationType(string type)
        {
            if (type == "" || type == "void")
                return "Smp::Publication::Uuid_void";

            return "Smp::PTK_" + type.Replace("Smp::", "");
        }
        public static string GetTypeUuid(string type)
        {
            if (type == "" || type == "void")
                return "Smp::Publication::Uuid_void";

            return "Smp::Uuid_" + type.Replace("Smp::", "");
        }
        public static string GetTagValue(EA.Collection tags, string tagName)
        {
            try
            {
                TaggedValue tag = tags.GetByName(tagName) as TaggedValue;
                return tag.Value;
            }
            catch
            {
                try
                {
                    AttributeTag tag = tags.GetByName(tagName) as AttributeTag;
                    return tag.Value;
                }
                catch
                {
                    return "";
                }
            }
        }
        public static string GetInitValuebByType(string type)
        {
            if (type == "Smp::Bool")
                return "false";
            else if (type == "Smp::Int8"
                    || type == "Smp::UInt8"
                    || type == "Smp::Int16"
                    || type == "Smp::UInt16"
                    || type == "Smp::Int32"
                    || type == "Smp::UInt32"
                    || type == "Smp::Int64"
                    || type == "Smp::UInt64"
                    || type == "Smp::Duration"
                    || type == "Smp::DateTime")
                return "0";
            else if (type == "Smp::Float32" || type == "Smp::Float64")
                return "0.0";
            else if (type == "Smp::Char8")
                return "'\0'";
            else if (type == "Smp::String8")
                return "NULL";

            return "\"\"";
        }
        static public string MultiplicityToUpperBound(string multiplicity)
        {
            string upperBound = "";
            if (multiplicity != "")
            {
                string[] mult = multiplicity.Replace("..", "#").Split('#');
                string lower = mult[0].TrimStart('[');
                if (mult.Count() > 1)
                    upperBound = mult[1].TrimEnd(']').Replace("*", "-1");
                else
                    upperBound = lower;
            }

            return upperBound;
        }
    }
}
