#include "CompositeLib.h"

#include "Model1_Level1.h"
#include "Model1_Level2.h"
#include "Model2_Level1.h"
#include "Model2_Level2.h"
#include "Model3_Level1.h"
#include "RootModel.h"
#include "Smp/Management/IManagedSimulator.h"
#include "Sb/Factory.h"

#include <vector>

/**
 * vector to store the factory objects until the end of DLL use.
 */
std::vector<Smp::IFactory*> m_factories;

/**
 * Initialise function for dynamic package.
 * @param simulator simulation environment reference
 * @param typeRegistry type register reference
 */
bool Initialise(Smp::ISimulator* simulator,
		Smp::Publication::ITypeRegistry* typeRegistry)
{
	Smp::Management::IManagedSimulator *sim = dynamic_cast<Smp::Management::IManagedSimulator*>(simulator);
	if(!sim)
		return false;

	// Catalogue::Model1_Level1 = {9DA9886F-56C3-4511-8251-53DB7B2ACE54}
	SB::Factory< Catalogue::Model1_Level1 >* factoryModel1_Level1
		= new SB::Factory< Catalogue::Model1_Level1 >("Model1_Level1", "Factory for Model1_Level1", Catalogue::Uuid_Model1_Level1, Catalogue::Uuid_Model1_Level1);
	m_factories.push_back(factoryModel1_Level1);
	sim->RegisterFactory(factoryModel1_Level1);

	// Catalogue::Model1_Level2 = {FA49988D-1C0B-4088-AFCB-3E6405EA99A0}
	SB::Factory< Catalogue::Model1_Level2 >* factoryModel1_Level2
		= new SB::Factory< Catalogue::Model1_Level2 >("Model1_Level2", "Factory for Model1_Level2", Catalogue::Uuid_Model1_Level2, Catalogue::Uuid_Model1_Level2);
	m_factories.push_back(factoryModel1_Level2);
	sim->RegisterFactory(factoryModel1_Level2);

	// Catalogue::Model2_Level1 = {500CE014-0068-44f1-82E2-8088D83D68AC}
	SB::Factory< Catalogue::Model2_Level1 >* factoryModel2_Level1
		= new SB::Factory< Catalogue::Model2_Level1 >("Model2_Level1", "Factory for Model2_Level1", Catalogue::Uuid_Model2_Level1, Catalogue::Uuid_Model2_Level1);
	m_factories.push_back(factoryModel2_Level1);
	sim->RegisterFactory(factoryModel2_Level1);

	// Catalogue::Model2_Level2 = {72A03B2B-AFE2-492b-80F2-924C81537E72}
	SB::Factory< Catalogue::Model2_Level2 >* factoryModel2_Level2
		= new SB::Factory< Catalogue::Model2_Level2 >("Model2_Level2", "Factory for Model2_Level2", Catalogue::Uuid_Model2_Level2, Catalogue::Uuid_Model2_Level2);
	m_factories.push_back(factoryModel2_Level2);
	sim->RegisterFactory(factoryModel2_Level2);

	// Catalogue::Model3_Level1 = {6A4A8248-CD3F-4979-A427-9A2A9CF7B4DE}
	SB::Factory< Catalogue::Model3_Level1 >* factoryModel3_Level1
		= new SB::Factory< Catalogue::Model3_Level1 >("Model3_Level1", "Factory for Model3_Level1", Catalogue::Uuid_Model3_Level1, Catalogue::Uuid_Model3_Level1);
	m_factories.push_back(factoryModel3_Level1);
	sim->RegisterFactory(factoryModel3_Level1);

	// Catalogue::RootModel = {FBFA7FA6-0D14-4ae7-AE8B-EE9E9D8E2FE5}
	SB::Factory< Catalogue::RootModel >* factoryRootModel
		= new SB::Factory< Catalogue::RootModel >("RootModel", "Factory for RootModel", Catalogue::Uuid_RootModel, Catalogue::Uuid_RootModel);
	m_factories.push_back(factoryRootModel);
	sim->RegisterFactory(factoryRootModel);

	return true;
}

/**
 * Finalise function for dynamic package.
 */
bool Finalise()
{
	if(!m_factories.size())
		return false;

	for(unsigned int i=0; i < m_factories.size(); i++)
		delete(m_factories[i]);

	m_factories.clear();

	return true;
}

