#include "ModelFields.h"
#include "Sb/EntryPoint.h"

// Unique identifier of type ModelFields
// UUID: 378B9821-00B9-4301-8A61-1B5260A0E3E1
const SB::Uuid Test::Uuid_ModelFields("378B9821-00B9-4301-8A61-1B5260A0E3E1");

Test::ModelFields::ModelFields()
{
	_Init();
}

Test::ModelFields::ModelFields(Smp::String8 name, Smp::String8 description, Smp::IComposite* parent)
	throw (Smp::Management::IManagedObject::InvalidObjectName)
  : SB::Management::ManagedModel(name, description, parent)
{
	_Init();
}

Test::ModelFields::~ModelFields()
{
	try {
		_Destroy();
	} catch (...) {
		m_simulator->GetLogger()->Log(
		this,
		"Exception during destruction of model 'SASModel'",
		::Smp::Services::LMK_Error);
	}
}
void Test::ModelFields::_Init()
{

	//////// Entry Points

	//////// References

	//////// Containers

	//////// Initialising fields to zero values
	this->simpleField1 = 0;
	this->simpleField2 = false;
	this->simpleField3 = NULL;
	this->simpleField4 = 0;
	this->simpleArray1[0] = 0;
	this->simpleArray2[0] = 0.0;
	this->simpleArray2[1] = 0.0;
	this->simpleArray3[0] = 0.0;
	this->simpleArray3[1] = 0.0;
	this->simpleArray3[2] = 0.0;
	this->simpleArray4[0] = 0;
	this->simpleArray4[1] = 0;
	this->simpleArray4[2] = 0;
	this->simpleArray4[3] = 0;
	this->inputField1 = 0;
	this->inputField2 = NULL;
	this->inputField3[0] = 0;
	this->inputField4[0] = 0.0;
	this->inputField4[1] = 0.0;
	this->inputField4[2] = 0.0;
	this->outputField1 = false;
	this->outputField2 = 0;
	this->outputField3[0] = 0.0;
	this->outputField3[1] = 0.0;
	this->outputField4[0] = 0;
	this->outputField4[1] = 0;
	this->outputField4[2] = 0;
	this->outputField4[3] = 0;
	//////// Initialising fields to initial values
	this->simpleField2 = true;
	this->simpleField3 = "Value";
	this->simpleArray1[0] = 1;
	this->simpleArray2[0] = 1.;
	this->simpleArray2[1] = 2.;
	this->simpleArray3[0] = 1.;
	this->simpleArray3[1] = 2.;
	this->simpleArray3[2] = 3.;
	this->simpleArray4[0] = 1;
	this->simpleArray4[1] = 2;
	this->simpleArray4[2] = 3;
	this->simpleArray4[3] = 4;
	this->inputField4[0] = 3.1;
	this->inputField4[1] = 3.2;
	this->inputField4[2] = 3.3;
	this->outputField2 = 100;
}

void Test::ModelFields::_Destroy()
{
}

/// ------------------------------------------------------
/// --------------------- Operations ---------------------


/// ------------------------------------------------------
/// --------------------- Entry Points ---------------------
/// --------------------------------------------------------
/// --------------------- IModel ---------------------
void Test::ModelFields::Configure(Smp::Services::ILogger* logger) throw (Smp::IModel::InvalidModelState)
{
	// User code here

	// At the end call method from base class
	SB::Model::Configure(logger);
}

void Test::ModelFields::Connect(Smp::ISimulator* simulator) throw (Smp::IModel::InvalidModelState)
{
	// User code here

	// At the end call method from base class
	SB::Model::Connect(simulator);
}

void Test::ModelFields::Publish(Smp::IPublication* receiver) throw (Smp::IModel::InvalidModelState)
{
	// Temp reference to publish operation's parameters
	Smp::Publication::IPublishOperation* operation = NULL;

	// Call base implementation first
	SB::Model::Publish(receiver);

	// ----- Publish Fields
	// simpleField1
	receiver->PublishField(
		"simpleField1",
		"Simple Field1",
		&simpleField1,
		Smp::VK_None,
		true,
		false,
		false);

	// simpleField2
	receiver->PublishField(
		"simpleField2",
		"",
		&simpleField2,
		Smp::VK_None,
		true,
		false,
		false);

	// simpleField3
	receiver->PublishField(
		"simpleField3",
		"",
		static_const<void*>(&simpleField3),
		Smp::PTK_String8,
		Smp::VK_Debug,
		false,
		false,
		false);

	// simpleField4
	receiver->PublishField(
		"simpleField4",
		"",
		static_const<void*>(&simpleField4),
		Smp::PTK_Duration,
		Smp::VK_None,
		true,
		false,
		false);

	// simpleArray1
	receiver->PublishArray(
		"simpleArray1",
		"",
		1,
		static_cast<void*>(simpleArray1),
		Smp::PTK_Int16,
		Smp::VK_None,
		true,
		false,
		false);

	// simpleArray2
	receiver->PublishArray(
		"simpleArray2",
		"",
		2,
		static_cast<void*>(simpleArray2),
		Smp::PTK_Float32,
		Smp::VK_None,
		true,
		false,
		false);

	// simpleArray3
	receiver->PublishArray(
		"simpleArray3",
		"",
		3,
		static_cast<void*>(simpleArray3),
		Smp::PTK_Float64,
		Smp::VK_Expert,
		true,
		false,
		false);

	// simpleArray4
	receiver->PublishArray(
		"simpleArray4",
		"",
		4,
		static_cast<void*>(simpleArray4),
		Smp::PTK_Int64,
		Smp::VK_All,
		false,
		false,
		false);

	// inputField1
	receiver->PublishField(
		"inputField1",
		"",
		&inputField1,
		Smp::VK_None,
		true,
		true,
		false);

	// inputField2
	receiver->PublishField(
		"inputField2",
		"",
		&inputField2,
		Smp::VK_Debug,
		false,
		true,
		false);

	// inputField3
	receiver->PublishArray(
		"inputField3",
		"",
		1,
		static_cast<void*>(inputField3),
		Smp::PTK_Int16,
		Smp::VK_None,
		true,
		true,
		false);

	// inputField4
	receiver->PublishArray(
		"inputField4",
		"",
		3,
		static_cast<void*>(inputField4),
		Smp::PTK_Float64,
		Smp::VK_Expert,
		false,
		true,
		false);

	// outputField1
	receiver->PublishField(
		"outputField1",
		"",
		&outputField1,
		Smp::VK_None,
		true,
		false,
		true);

	// outputField2
	receiver->PublishField(
		"outputField2",
		"",
		&outputField2,
		Smp::VK_None,
		true,
		false,
		true);

	// outputField3
	receiver->PublishArray(
		"outputField3",
		"",
		2,
		static_cast<void*>(outputField3),
		Smp::PTK_Float32,
		Smp::VK_None,
		true,
		false,
		true);

	// outputField4
	receiver->PublishArray(
		"outputField4",
		"",
		4,
		static_cast<void*>(outputField4),
		Smp::PTK_Int64,
		Smp::VK_All,
		false,
		false,
		true);


}

